<?php
// $Id: sunmailer.admin.inc,v 1.1.2.3 2009/12/28 16:59:41 mwacker Exp $

/**
 * @file
 * Configures the settings for Sunmailer.
 */

/**
 * Generates a form for PHPMailer settings.
 */
function sunmailer_admin_settings() {
  $form = array();
  // Important settings.
  $form['sunmailer_phpmailer_path'] = array(
    '#type' => 'textfield',
    '#title' => t('PHPMailer Path'),
    '#description' => t('Absolute path to the directory containing class.phpmailer.php'),
    '#default_value' => variable_get('sunmailer_phpmailer_path', ''),
  );
  $form['sunmailer_test_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Test Email'),
    '#description' => t('Email address to receive test emails'),
    '#default_value' => variable_get('sunmailer_test_email', ''),
  );
  $form['button'] = array(
    '#type' => 'submit',
    '#value' => t('Send Test Email'),
  );

  // Basic settings.
  $form['basic'] = array(
    '#type' => 'fieldset',
    '#title' => t('Basic Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['basic']['sunmailer_from_name'] = array(
    '#type' => 'textfield',
    '#title' => t('From Name'),
    '#description' => t('Name of the sender'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_name', ''),
  );
  $form['basic']['sunmailer_from_email'] = array(
    '#type' => 'textfield',
    '#title' => t('From Email'),
    '#description' => t('Email of the sender'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_from_email', ''),
  );
  $form['basic']['sunmailer_host'] = array(
    '#type' => 'textfield',
    '#title' => t('Host'),
    '#description' => t('URL or IP address for the server sending the message'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_host', 'localhost'),
  );
  $form['basic']['sunmailer_port'] = array(
    '#type' => 'textfield',
    '#title' => t('Port'),
    '#description' => t('Port the server sends the message on'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_port', 25),
  );

  // Mail settings.
  $form['mail'] = array(
    '#type' => 'fieldset',
    '#title' => t('Mail Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['mail']['sunmailer_method'] = array(
    '#type' => 'radios',
    '#title' => t('Mailing Method'),
    '#description' => t('Selects the method used to send the newsletter'),
    '#required' => TRUE,
    '#options' => array(
      'mail' => 'Mail',
      'sendmail' => 'Sendmail',
      'smtp' => 'SMTP',
    ),
    '#default_value' => variable_get('sunmailer_method', 'mail'),
  );
  // Sendmail settings.
  $form['mail']['sunmailer_sendmail_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Sendmail Path'),
    '#description' => t('Absolute path to the sendmail program'),
    '#default_value' => variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail'),
  );
  // SMTP settings.
  $form['mail']['sunmailer_smtp_auth'] = array(
    '#type' => 'checkbox',
    '#title' => t('SMTP Authentication'),
    '#description' => t('Check this to enable SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_auth', FALSE),
  );
  $form['mail']['sunmailer_smtp_user'] = array(
    '#type' => 'textfield',
    '#title' => t('SMTP Username'),
    '#description' => t('Username for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_user', ''),
  );
  $form['mail']['sunmailer_smtp_pass'] = array(
    '#type' => 'textfield',
    '#title' => t('SMTP Password'),
    '#description' => t('Password for SMTP authentication'),
    '#default_value' => variable_get('sunmailer_smtp_pass', ''),
  );
  $form['mail']['sunmailer_smtp_secure'] = array(
    '#type' => 'radios',
    '#title' => t('SMTP Security'),
    '#description' => t('Sets the encryption used for sending emails'),
    '#options' => array(
      '' => t('None'),
      'ssl' => 'SSL',
      'tls' => 'TLS',
    ),
    '#default_value' => variable_get('sunmailer_smtp_secure', ''),
  );

  // Advanced settings.
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['sunmailer_delay'] = array(
    '#type' => 'textfield',
    '#title' => t('Next Mailing Delay'),
    '#description' => t('Number of minutes to wait before allowing another mailing'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_delay', 5),
  );
  $form['advanced']['sunmailer_retries'] = array(
    '#type' => 'textfield',
    '#title' => t('Resend Attempts'),
    '#description' => t('Number of times to retry if the message fails to be sent'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_retries', 2),
  );
  $form['advanced']['sunmailer_max_recipients'] = array(
    '#type' => 'textfield',
    '#title' => t('Maximum Number of Recipients'),
    '#description' => t('Emails with more than the maximum will be split into multiple emails.  Use 0 for no limit.'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_max_recipients', 0),
  );
  $form['advanced']['sunmailer_word_wrap'] = array(
    '#type' => 'textfield',
    '#title' => t('Word Wrap'),
    '#description' => t('Maximum number of text characters on one line'),
    '#required' => TRUE,
    '#default_value' => variable_get('sunmailer_word_wrap', 80),
  );
  $form['advanced']['sunmailer_char_set'] = array(
    '#type' => 'textfield',
    '#title' => t('Character Set'),
    '#description' => t('Character encoding to use for emails'),
    '#required' => TRUE,
    '#default_value' => 'utf-8',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for PHPMailer settings form.
 */
function sunmailer_admin_settings_validate($form, &$form_state) {
  $values = $form_state['values'];
  // Validate top-level settings and email test button.
  if (!file_exists("{$values['sunmailer_phpmailer_path']}/class.phpmailer.php")) {
    form_set_error('sunmailer_phpmailer_path', t('Cannot find PHPMailer.'));
  }
  if ($form_state['clicked_button']['#value'] == 'Send Test Email') {
    if (!$values['sunmailer_test_email']) {
      form_set_error('sunmailer_test_email', t('Test Email is required to send a test email.'));
    }
  }
  // Validate basic settings.
  if (!is_numeric($values['sunmailer_port']) || $values['sunmailer_port'] < 0) {
    form_set_error('sunmailer_port', t('Port must be a positive integer.'));
  }
  // Validate mail settings.
  switch ($values['sunmailer_method']) {
  case 'sendmail':
    if (!$values['summailer_sendmail_path']) {
      form_set_error('sunmailer_sendmail_path', t('Sendmail Path is required if using Sendmail.'));
    }
    break;
  case 'smtp':
    if ($values['sunmailer_smtp_auth']) {
      if (!$values['sunmailer_smtp_user']) {
        form_set_error('sunmailer_smtp_user', t('SMTP Username is required if using SMTP with authentication.'));
      }
      if (!$values['sunmailer_smtp_pass']) {
        form_set_error('sunmailer_smtp_pass', t('SMTP Password is required if using SMTP with authentication.'));
      }
    }
    break;
  }
  // Validate advanced settings.
  if (!is_numeric($values['sunmailer_delay']) || $values['sunmailer_delay'] < 0) {
    form_set_error('sunmailer_delay', t('Next Mailing Delay must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_retries']) || $values['sunmailer_retries'] < 0) {
    form_set_error('sunmailer_retries', t('Resend Attempts must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_max_recipients']) || $values['sunmailer_max_recipients'] < 0) {
    form_set_error('sunmailer_max_recipients', t('Maximum Number of Recipients must be a positive integer.'));
  }
  if (!is_numeric($values['sunmailer_word_wrap']) || $values['sunmailer_word_wrap'] < 0) {
    form_set_error('sunmailer_word_wrap', t('Word Wrap must be a positive integer.'));
  }
}

/**
 * Submission callback for PHPMailer settings form.
 */
function sunmailer_admin_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  // Set top-level settings.
  variable_set('sunmailer_phpmailer_path', $values['sunmailer_phpmailer_path']);
  variable_set('sunmailer_test_email', $values['sunmailer_test_email']);
  // Set basic settings.
  variable_set('sunmailer_from_name', $values['sunmailer_from_name']);
  variable_set('sunmailer_from_email', $values['sunmailer_from_email']);
  variable_set('sunmailer_host', $values['sunmailer_host']);
  variable_set('sunmailer_port', $values['sunmailer_port']);
  // Set mail settings.
  variable_set('sunmailer_method', $values['sunmailer_method']);
  variable_set('sunmailer_sendmail_path', $values['sunmailer_sendmail_path']);
  variable_set('sunmailer_smtp_auth', $values['sunmailer_smtp_auth']);
  variable_set('sunmailer_smtp_user', $values['sunmailer_smtp_user']);
  variable_set('sunmailer_smtp_pass', $values['sunmailer_smtp_pass']);
  variable_set('sunmailer_smtp_secure', $values['sunmailer_smtp_secure']);
  // Set advanced settings.
  variable_set('sunmailer_delay', $values['sunmailer_delay']);
  variable_set('sunmailer_retries', $values['sunmailer_retries']);
  variable_set('sunmailer_max_recipients', $values['sunmailer_max_recipients']);
  variable_set('sunmailer_word_wrap', $values['sunmailer_word_wrap']);
  variable_set('sunmailer_char_set', $values['sunmailer_char_set']);
  // Send a test email if requested.
  if ($form_state['clicked_button']['#value'] == 'Send Test Email') {
    $mailer = sunmailer_get_mailer();
    $mailer->AddAddress(variable_get('sunmailer_test_email', ''));
    $mailer->Subject = 'SunMailer Test';
    $mailer->Body = print_r($mailer, TRUE);
    $mailer->Send();
  }
}

/**
 * Builds the form for one format.
 */
function _sunmailer_format_form($xid, $format) {
  // Generate the title and XSL name for the format.
  $new = $xid == 'new';
  $type = $format['html'] ? 'HTML' : 'text';
  if ($new) {
    $title = 'New Format';
  }
  else {
    $xsl_name = db_result(db_query("SELECT filename FROM {files} WHERE fid = %d", $format['fid']));
    $title = $format['name'];
    $title .= $format['custom'] ? '' : t(' (default @type format)', array('@type' => $type));
  }
  // Build the form.
  $form = array(
    '#type' => 'fieldset',
    '#title' => t($title),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => !$new,
    '#default_value' => $format['name'],
  );
  $form['html'] = array(
    '#type' => 'radios',
    '#title' => t('Format Type'),
    '#options' => array(
      FALSE => t('Text'),
      TRUE => t('HTML'),
    ),
    '#default_value' => $new ? TRUE : $format['html'],
    '#disabled' => !$new,
  );
  $form['file'] = array(
    '#type' => 'file',
    '#title' => t('XSL'),
    '#description' => t('XSL file used to build this format from XML content'),
  );
  if (!$new) {
    $form['xsl_name'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#value' => '<strong>' . t('Current XSL File:') . "</strong> $xsl_name",
      '#suffix' => '</p>',
    );
  }
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $new ? 0 : $format['weight'],
  );
  if (!$new) {
    if (!$format['custom']) {
      if (($format['fid'] != variable_get('sunmailer_default_html_fid', 0)) && ($format['fid'] != variable_get('sunmailer_default_text_fid', 0))) {
        $form['reset'] = array(
          '#type' => 'checkbox',
          '#title' => t('Reset XSL file'),
        );
      }
    }
    else {
      $form['delete'] = array(
        '#type' => 'checkbox',
        '#title' => t('Delete'),
        '#description' => t('Users with this format will be switched to the default @type format', array('@type' => $type)),
      );
    }
  }
  return $form;
}

/**
 * Generates a form for newsletter formats.
 */
function sunmailer_admin_formats() {
  $form = array();
  $form['#tree'] = TRUE;
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  // Build forms for formats.
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $format) {
    $form[$xid] = _sunmailer_format_form($xid, $format);
  }
  $form['new'] = _sunmailer_format_form('new', array());
  // Build the form for choosing a default format.
  $format_options = array();
  foreach ($formats as $xid => $format) {
    $format_options[$xid] = $format['name'];
  }
  $format_options['new'] = 'New Format';
  $form['default_format'] = array(
    '#type' => 'radios',
    '#title' => t('Default Format'),
    '#description' => t('Choose the format new users should default to'),
    '#options' => $format_options,
    '#default_value' => variable_get('sunmailer_default_format', SUNMAILER_HTML_FORMAT),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for XSLT file.
 */
function sunmailer_is_xslt_file(&$file) {
  $doc = new DOMDocument();
  return $doc->load($file->filepath) ? array() : array(t('This is not a valid XSL file.'));
}

/**
 * Validation callback for newsletter formats form.
 */
function sunmailer_admin_formats_validate($form, &$form_state) {
  $values = $form_state['values'];
  $validators = array(
    'sunmailer_is_xslt_file' => array(),
  );
  // Validate formats.
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $v) {
    if (!$format['delete']) {
      $form_state['values'][$xid]['xsl'] = file_save_upload($xid, $validators, file_directory_path());
    }
  }
  if (trim($values['new']['name'])) {
    $xslt = file_save_upload('new', $validators, file_directory_path());
    if (!$xslt) {
      form_set_error('new][file', 'XSL file required');
    }
    $form_state['values']['new']['xsl'] = $xslt;
  }
  // Check the default format.
  $default_format = $values[$values['default_format']];
  if (!trim($default_format['name'])) {
    form_set_error('default_format', t('No new format exists.'));
  }
  if ($default_format['delete']) {
    form_set_error('default_format', t('This format is being deleted; select a new default format.'));
  }
}

/**
 * Submission callback for newsletter formats form.
 */
function sunmailer_admin_formats_submit($form, &$form_state) {
  $values = $form_state['values'];
  $formats = sunmailer_get_formats();
  foreach ($formats as $xid => $v) {
    $format = $values[$xid];
    // Find the file for the current XSL file.
    $fid = db_result(db_query("SELECT fid FROM {sunmailer_formats} WHERE xid = %d", $xid));
    $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $fid));
    // Delete custom format.
    if ($format['delete']) {
      $new_xid = $formats[$xid]['html'] ? SUNMAILER_HTML_FORMAT : SUNMAILER_TEXT_FORMAT;
      db_query("UPDATE {sunmailer_user_format} SET xid = %d WHERE xid = %d", $new_xid, $xid);
      file_delete($path);
      db_query("DELETE FROM {sunmailer_formats} WHERE xid = %d", $xid);
    }
    // Update custom format.
    else {
      if ($format['reset']) {
        db_query("DELETE FROM {files} WHERE fid = %d", $fid);
        file_delete($path);
        $fid = variable_get(($xid == SUNMAILER_HTML_FORMAT) ? 'sunmailer_default_html_fid' : 'sunmailer_default_text_fid', 0);
      }
      else {
        $file = $format['xsl'];
        if ($file) {
          if (($fid != variable_get('sunmailer_default_html_fid', 0)) && ($fid != variable_get('sunmailer_default_text_fid', 0))) {
            db_query("DELETE FROM {files} WHERE fid = %d", $fid);
            file_delete($path);
          }
          file_set_status($file, FILE_STATUS_PERMANENT);
          $fid = $file->fid;
        }
      }
      $args = array($format['name'], $fid, $format['weight'], $xid);
      db_query("UPDATE {sunmailer_formats} SET name = '%s', fid = %d, weight = %d WHERE xid = %d", $args);
    }
  }
  // Check for a new format.
  $format = $values['new'];
  $name = $format['name'];
  if (trim($name)) {
    $file = $format['xsl'];
    file_set_status($file, FILE_STATUS_PERMANENT);
    $new_format = new stdClass();
    $new_format->name = $name;
    $new_format->html = $format['html'];
    $new_format->fid = $file->fid;
    $new_format->weight = $format['weight'];
    drupal_write_record('sunmailer_formats', $new_format);
    if ($values['default_format'] == 'new') {
      $values['default_format'] = $new_format->xid;
    }
  }
  // Update default format.
  variable_set('sunmailer_default_format', (int)$values['default_format']);
}

/**
 * Autocomplete callback for views.
 */
function sunmailer_autocomplete_views($string = '') {
  $result = db_query_range("SELECT name FROM {views_view} WHERE name LIKE '%s%%' AND base_table = 'node'", $string, 0, 10);
  $matches = array();
  while ($match = db_result($result)) {
    $matches[$match] = $match;
  }
  drupal_json($matches);
}

/**
 * Builds the form for one section.
 */
function _sunmailer_section_form($section, $new) {
  $view = db_result(db_query("SELECT name FROM {views_view} WHERE vid = %d AND base_table = 'node'", $section['vid']));
  $form = array(
    '#type' => 'fieldset',
    '#title' => $new ? t('New Section') : $section['name'],
    '#collapsible' => !$new,
    '#collapsed' => !$new,
  );
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#required' => !$new,
    '#default_value' => $section['name'],
  );
  $form['view'] = array(
    '#type' => 'textfield',
    '#title' => t('View'),
    '#description' => t('View containing the content for this section'),
    '#required' => !$new,
    '#default_value' => $view,
    '#autocomplete_path' => 'sunmailer/autocomplete/views',
  );
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $new ? 0 : $section['weight'],
  );
  if (!$new) {
    $form['delete'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete'),
      '#description' => t('Check this to delete this section'),
    );
  }
  return $form;
}

/**
 * Generates a form for newsletter sections.
 */
function sunmailer_admin_sections() {
  $form = array();
  $form['#tree'] = TRUE;
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $section) {
    $form[$sid] = _sunmailer_section_form($section, FALSE);
  }
  $form['new'] = _sunmailer_section_form(array(), TRUE);
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Checks that the view exists for one section.
 */
function _sunmailer_section_validate($id, $section, $new) {
  if ($new && !trim($section['name'])) {
    return;
  }
  if (!$new && $section['delete']) {
    return;
  }
  if (!db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']))) {
    form_set_error("$id][view", t('View does not exist or is not a node view.'));
  }
}

/**
 * Validation callback for newsletter settings form.
 */
function sunmailer_admin_sections_validate($form, &$form_state) {
  $values = $form_state['values'];
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $v) {
    _sunmailer_section_validate($sid, $values[$sid], FALSE);
  }
  _sunmailer_section_validate('new', $values['new'], TRUE);
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_admin_sections_submit($form, &$form_state) {
  $values = $form_state['values'];
  $sections = sunmailer_get_sections();
  foreach ($sections as $sid => $v) {
    $section = $values[$sid];
    // Delete section, unsubscribe users from it.
    if ($section['delete']) {
      db_query("DELETE FROM {sunmailer_sections} WHERE sid = %d", $sid);
      db_query("DELETE FROM {sunmailer_user_sections} WHERE sid = %d", $sid);
    }
    // Update section.
    else {
      $vid = db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']));
      $args = array(trim($section['name']), $vid, $section['weight'], $sid);
      db_query("UPDATE {sunmailer_sections} SET name = '%s', vid = %d, weight = %d WHERE sid = %d", $args);
    }
  }
  // Check for a new section.
  $section = $values['new'];
  $name = trim($section['name']);
  if ($name) {
    $vid = db_result(db_query("SELECT vid FROM {views_view} WHERE name = '%s' AND base_table = 'node'", $section['view']));
    $args = array($name, $vid, $section['weight']);
    db_query("INSERT INTO {sunmailer_sections}(name, vid, weight) VALUES ('%s', %d, %d)", $args);
  }
}
