<?php
// $Id: sunmailer.mail.inc,v 1.1.2.6 2009/12/28 23:51:25 mwacker Exp $

/**
 * @file
 * Sends emails.
 */
 
 /**
 * Generates a form for newsletter settings.
 */
function sunmailer_mail_settings() {
  $form = array();
  $form['sunmailer_subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('Subject line for the newsletter'),
    '#default_value' => variable_get('sunmailer_subject', ''),
  );
  $form['sunmailer_message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#description' => t('This message will appear at the top of the newsletter'),
    '#default_value' => variable_get('sunmailer_message', ''),
  );
  $cutoff = variable_get('sunmailer_cutoff', '');
  $form['sunmailer_cutoff'] = array(
    '#type' => 'textfield',
    '#title' => t('Cutoff Date'),
    '#description' => t('The email newsletter will exclude content published before this date'),
    '#default_value' => $cutoff ? format_date($cutoff, 'custom', 'Y-m-d H:i:s O') : '',
  );
  $form['sunmailer_submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Validation callback for newsletter settings form.
 */
function sunmailer_mail_settings_validate($form, &$form_state) {
  $values = $form_state['values'];
  if (!empty($values['sunmailer_cutoff']) && !strtotime($values['sunmailer_cutoff'])) {
    form_set_error('cutoff', t('Invalid date format.'));
  }
}

/**
 * Submission callback for newsletter settings form.
 */
function sunmailer_mail_settings_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['sunmailer_subject']) {
    variable_set('sunmailer_subject', $values['sunmailer_subject']);
  }
  else {
    variable_del('sunmailer_subject');
  }
  variable_set('sunmailer_message', $values['sunmailer_message']);
  variable_set('sunmailer_cutoff', strtotime($values['sunmailer_cutoff']));
}

/**
 * Generates a form to send the newsletter.
 */
function sunmailer_mail_newsletter() {
  // Build a newsletter with all sections in the default format.
  require_once 'sunmailer.xml.inc';
  $xml = sunmailer_xml_newsletter(array_keys(sunmailer_get_sections()));
  if (!$xml) {
    drupal_set_message(t('The email newsletter currently has no content.'));
    return;
  }
  $formats = sunmailer_get_formats();
  $xid = variable_get('sunmailer_text_default', SUNMAILER_TEXT_FORMAT);
  $preview = htmlentities(sunmailer_transform_xml($xml, $formats[$xid]));
  // Display the last time the newsletter was sent.
  $last_sent = variable_get('sunmailer_lastsent', 0);
  if ($last_sent) {
    drupal_set_message(t('The newsletter was last sent @date.', array('@date' => date('l, F j, Y g:i:s A', $last_sent))));
  }
  // Build the form.
  $form = array();
  $form['text'] = array(
    '#prefix' => '<div>',
    '#value' => nl2br($preview),
    '#suffix' => '</div><hr />',
  );
  $form['save_message'] = array(
    '#type' => 'checkbox',
    '#title' => t('Save the message at the top for future newsletters'),
    '#default_value' => FALSE,
  );
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send the email newsletter'),
    '#default_value' => FALSE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for form to send the newsletter.
 */
function sunmailer_mail_newsletter_validate($form, &$form_state) {
  if (!$form_state['values']['confirm']) {
    form_set_error('confirm', t('Cannot send without confirmation.'));
  }
  $delay = variable_get('sunmailer_delay', 5);
  if ((time() - variable_get('sunmailer_lastsent', 0)) < ($delay * 60)) {
    form_set_error('confirm', t('You must wait @delay minutes before sending Sunmailer again.', array('@delay' => $delay)));
  }
}

/**
 * Submission callback for form to send the newsletter.
 */
function sunmailer_mail_newsletter_submit($form, &$form_state) {
  require_once 'sunmailer.xml.inc';
  // Set time-based variables.
  watchdog('mail', t('Sending the email newsletter'));
  variable_set('sunmailer_lastsent', time());
  variable_set('sunmailer_cutoff', time());
  // Retrieve sections, format for all users.
  $result = db_query("SELECT uid, sid FROM {sunmailer_user_sections} ORDER BY uid, sid ASC");
  $user_sections = array();
  while ($row = db_fetch_object($result)) {
    $user_sections[$row->uid][] = $row->sid;
  }
  $result = db_query("SELECT u.uid, u.mail, uf.xid FROM {users} u, {sunmailer_user_format} uf WHERE u.uid = uf.uid");
  $user_format = array();
  while ($row = db_fetch_object($result)) {
    $user_format[$row->uid] = $row->xid;
    $user_mail[$row->uid] = $row->mail;
  }
  $subscriptions = array();
  foreach ($user_sections as $uid => $sids) {
    $subscriptions[implode(',', $sids)][$user_format[$uid]][$uid] = $user_mail[$uid];
  }
  // Build the newsletters for each combination of sections.
  $formats = sunmailer_get_formats();
  $alt_xid = variable_get('sunmailer_text_default', SUNMAILER_TEXT_FORMAT);
  $alt_format = $formats[$alt_xid];
  $mailer = sunmailer_get_mailer();
  $subject = variable_get('sunmailer_subject', variable_get('site_name', t('SunMailer')));
  foreach ($subscriptions as $sid_list => $groups) {
    $sids = explode(',', $sid_list);
    $xml = sunmailer_xml_newsletter($sids);
    if (!$xml) {
      continue;
    }
    // Construct the newsletter in each format and send it.
    foreach ($groups as $xid => $addresses) {
      $format = $formats[$xid];
      $body = sunmailer_transform_xml($xml, $format);
      $is_html = $format['html'];
      if ($is_html) {
        $alt_notice = 'Note: You are viewing this as text because your email client does not support HTML.';
        $alt_body = "$alt_notice\n\n" . sunmailer_transform_xml($xml, $alt_format);
      }
      sunmailer_send_mail($mailer, $addresses, $subject, $body, $is_html, $alt_body);
    }
  }
  // Preserve message if asked.
  watchdog('mail', t('Sent the email newsletter'));
  if (!$form_state['values']['save_message']) {
    variable_del('sunmailer_message');
  }
  // Redirect.
  $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
}

/**
 * Creates a form to send a custom text email message
 */
function sunmailer_mail_custom(&$form_state = NULL) {
  $form = array();
  if (isset($form_state['storage']['preview'])) {
    $form['#prefix'] = $form_state['storage']['preview'];
  }
  $form['message'] = array(
    '#type' => 'fieldset',
    '#title' => t('Custom Message'),
    '#description' => t('Message to email to all subscribers'),
  );
  $form['message']['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => isset($form_state) ? $form_state['values']['subject'] : '',
  );
  $form['message']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Body'),
    '#default_value' => isset($form_state) ? $form_state['values']['body'] : '',
  );
  $form['confirm'] = array(
    '#type' => 'checkbox',
    '#title' => t('Confirm that you want to send this email'),
    '#default_value' => FALSE,
  );
  $form['preview'] = array(
    '#type' => 'submit',
    '#value' => t('Preview'),
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Send'),
  );
  return $form;
}

/**
 * Validation callback for the form to send a custom email.
 */
function sunmailer_mail_custom_validate($form, &$form_state) {
  if (!$form_state['values']['subject']) {
    form_set_error('subject', t('Subject is blank.'));
  }
  if (!$form_state['values']['body']) {
    form_set_error('body', t('Body is blank.'));
  }
  if (!$form_state['values']['confirm'] && $form_state['clicked_button']['#value'] != t('Preview')) {
    form_set_error('confirm', t('Cannot send without confirmation.'));
  }
}

/**
 * Submission callback for the form to send a custom email.
 */
function sunmailer_mail_custom_submit($form, &$form_state) {
  // Send the email.
  if ($form_state['clicked_button']['#value'] == 'Send') {
    watchdog('mail', t('Sending a custom email message'));
    // Get all subscribers.
    $result = db_query("SELECT DISTINCT(u.mail) FROM {users} u, {sunmailer_user_sections} us WHERE u.uid = us.uid");
    $addresses = array();
    while ($address = db_result($result)) {
      $addresses[] = $address;
    }
    // Send the email in the text format.
    $values = $form_state['values'];
    $mailer = sunmailer_get_mailer();
    sunmailer_send_mail($mailer, $addresses, check_plain($values['subject']), check_plain($values['body']));
    watchdog('mail', t('Sent a custom email message'));
    // Redirect.
    unset($form_state['storage']);
    $form_state['redirect'] = user_access('access site reports') ? 'admin/reports/dblog' : 'admin/settings/sunmailer';
  }
  // Store a preview of the email newsletter.
  else {
    $preview = "<h3>" . check_plain($form_state['values']['subject']) . "</h3>";
    $preview .= "<p>" . check_plain($form_state['values']['body']) . "</p>";
    $form_state['storage']['preview'] = $preview;
  }
}

/**
 * Generates subscription statistics for SunMailer.
 */
function get_sunmailer_stats() {
  // Get total subscribers.
  $stats['total'] = db_result(db_query("SELECT COUNT(DISTINCT(uid)) FROM {sunmailer_user_sections}"));
  $stats['splits'] = array();
  // Get subscribers by format.
  $stats['splits']['Format'] = array();
  $formats = db_query("SELECT f.name, COUNT(DISTINCT(uf.uid)) AS count FROM {sunmailer_user_format} uf, {sunmailer_formats} f, {sunmailer_user_sections} us WHERE uf.xid = f.xid AND uf.uid = us.uid GROUP BY f.xid");
  while ($row = db_fetch_object($formats)) {
    $stats['splits']['Format'][$row->name] = $row->count;
  }
  // Get subscribers by section.
  $stats['splits']['Section'] = array();
  $sections = db_query("SELECT s.name, COUNT(us.uid) AS count FROM {sunmailer_user_sections} us, {sunmailer_sections} s WHERE us.sid = s.sid GROUP BY s.sid");
  $stats['Section'] = array();
  while ($row = db_fetch_object($sections)) {
    $stats['splits']['Section'][$row->name] = $row->count;
  }
  return $stats;
}

/**
 * Displays a page of SunMailer statistics.
 */
function sunmailer_stats_page() {
  return theme('sunmailer_stats', get_sunmailer_stats());
}

/**
 * Default theme for Sunmailer statistics.
 */
function theme_sunmailer_stats($stats) {
  $html = '<h3>' . t('Total Subscribers') . '</h3>';
  $html .= '<ul><li>' . check_plain($stats['total']) . '</li></ul>';
  foreach ($stats['splits'] as $name => $split) {
    $html .= '<h4>' . t('By @name', array('@name' => $name)) . '</h4>';
    $html .= '<ul>';
    foreach ($split as $name => $count) {
      $html .= '<li>' . t('@name: @count', array('@name' => $name, '@count' => $count)) . '</li>';
    }
    $html .= '</ul>';
  }
  return $html;
}
