<?php
// $Id: sunmailer.xml.inc,v 1.1.2.5 2009/12/28 23:35:47 mwacker Exp $

/**
 * @file
 * Builds the XML newsletter and the XSL to transform it.
 */

/**
 * Generates the XML for a newsletter with selected sections.
 */
function sunmailer_xml_newsletter($sids) {
  static $cached = FALSE;
  static $no_section_title = FALSE;
  static $sections = array();
  // Build all sections once, then cache them.
  if (!$cached) {
    $cached = TRUE;
    $sections = sunmailer_get_sections();
    foreach ($sections as &$sectionn) { // not a typo, see PHP's foreach() documentation
      sunmailer_get_section_nodes($sectionn);
    }
    if (count($sections) == 1) {
      $no_section_title = TRUE;
    }
  }
  // Get XML for each section.
  $xml = '';
  foreach ($sections as $sid => $section) {
    if (in_array($sid, $sids)) {
      $xml .= sunmailer_xml_section($section, $no_section_title);
    }
  }
  // Return nothing if no sections have content.
  if (!$xml) {
    return NULL;
  }
  // Build newsletter XML.
  $xml = sunmailer_xml_header() . $xml . sunmailer_xml_footer();
  $doc = new DOMDocument();
  $doc->loadXML($xml);
  return $doc;
}

/**
 * Retrieves the nodes for an individual section.
 */
function sunmailer_get_section_nodes(&$section) {
  $view = views_get_view($section['vid']);
  $view->execute();
  $section['nodes'] = array();
  foreach ($view->result as $row) {
    $node = node_load($row->nid);
    if ($node->created >= variable_get('sunmailer_cutoff', 0)) {
      $section['nodes'][] = $node;
    }
  }
  return $section;
}

/**
 * Generates the XML for the header.
 */
function sunmailer_xml_header() {
  static $cached = FALSE;
  static $xml;
  if ($cached) {
    return $xml;
  }
  $cached = TRUE;
  // Retrieve the header information.
  $encoding = check_plain(variable_get('sunmailer_char_set', 'utf-8'));
  $title = check_plain(variable_get('sunmailer_subject', variable_get('site_name', t('SunMailer'))));
  global $base_url;
  $date = date('l, F j, Y');
  $message = check_plain(variable_get('sunmailer_message', ''));
  // Build the header XML.
  $xml  = '<?xml version="1.0" encoding="' . $encoding .'"?>';
  $xml .= '<newsletter>';
  $xml .= "<title><![CDATA[$title]]></title>";
  $xml .= "<link><![CDATA[$base_url]]></link>";
  $xml .= "<date><![CDATA[$date]]></date>";
  if (trim($message)) {
    $xml .= "<message><![CDATA[$message]]></message>";
  }
  return $xml;
}

/**
 * Generates the XML for a section.
 */
function sunmailer_xml_section($section, $no_title) {
  // Return nothing if the section has no content.
  if (!count($section['nodes'])) {
    return '';
  }
  // Retrieve the section information.
  $name = check_plain($section['name']);
  // Build the section XML.
  $xml = '<section>';
  if (!$no_title) {
    $xml .= "<name><![CDATA[$name]]></name>";
  }
  foreach ($section['nodes'] as $node) {
    $xml .= sunmailer_xml_content($node);
  }
  $xml .= '</section>';
  return $xml;
}

/**
 * Generates the XML for a content piece.
 */
function sunmailer_xml_content($node) {
  // Retrieve the content information.
  $title = check_plain($node->title);
  global $base_url;
  $link = $base_url . url("node/{$node->nid}");
  if ($node->uid) {
    $author = check_plain($node->name);
    $authorlink = $base_url . url("user/{$node->uid}");
  }
  $date = format_date($node->created);
  // Build the content XML.
  $xml = '<content>';
  $xml .= "<title><![CDATA[$title]]></title>";
  $xml .= "<link><![CDATA[$link]]></link>";
  if ($author) {
    $xml .= "<author><![CDATA[$author]]></author>";
    $xml .= "<authorlink><![CDATA[$authorlink]]></authorlink>";
  }
  $xml .= "<date><![CDATA[$date]]></date>";
  $xml .= "<teaser><![CDATA[{$node->teaser}]]></teaser>";
  $xml .= '</content>';
  return $xml;
}

/**
 * Generates the footer for the XML.
 */
function sunmailer_xml_footer() {
  static $cached = FALSE;
  static $xml;
  if ($cached) {
    return $xml;
  }
  $cached = TRUE;
  // Retrieve the footer information.
  global $base_url;
  $login_link = $base_url . url("user");
  $login_link = l($login_link, $login_link);
  $password_link = $base_url . url("user/password");
  $password_link = l($password_link, $password_link);
  $notice = "To change subscription options or unsubscribe, login to your account at $login_link. ";
  $notice .= "If you forgot your password or need a new one, reset it at $password_link.";
  // Build the footer XML.
  $xml = "<notice><![CDATA[$notice]]></notice>";
  $xml .= '</newsletter>';
  return $xml;
}

/**
 * Transforms XML into human-readable content.
 */
function sunmailer_transform_xml($xml, $format) {
  // Load XSL.
  $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $format['fid']));
  $doc = new DOMDocument();
  $doc->load($path);
  $xslt = new XSLTProcessor();
  $xslt->importStyleSheet($doc);
  // Transform XML, convert HTML to tag-less text for text formats.
  $newsletter = $xslt->transformToXML($xml);
  if (!$format['html']) {
    $newsletter = strip_tags($newsletter);
    $newsletter = html_entity_decode($newsletter);
  }
  return $newsletter;
}